##############################################
### Generation of Artificial Example Data  ###
##############################################
d = 1000 ### Number of Genes
gnames = paste("Gene.", 1:d, sep="")
n1 = 100 ### Number of Samples in Control Group
n2 = 100 ### Number of Samples in Disease Group
n = n1 + n2
mu1 = rlnorm(d, 0, 1)
mu2 = mu1
deg = sample(1:d, 100, replace=FALSE)
mu2[deg] = mu2[deg] * rlnorm(100, 0, sdlog=0.05)
hist(log2(mu1/mu2), cex.lab=1.5, cex.axis=1.5, xlab="log2 fold change", ylab="Frequency", main="") ### Verteilung der log2-Fold Changes
X = matrix(NA, d, n)
for (i in 1:n1) X[,i] = rnorm(d, mu1, 0.1 * mu1)
for (i in (n1+1):n) X[,i] = rnorm(d, mu2, 0.1 * mu2)
colnames(X) = c(paste("Control.", 1:n1, sep=""), paste("Disease.", (n1+1):n, sep="")) 
#heatmap(X)


#########################################
### Differentielle Expressionsanalyse ###
#########################################
P = rep(NA, d)
logFC = rep(NA, d)
for (i in 1:d) {
	#if (i%%10==0) print(100 * i/d) 
	P[i] = t.test(X[i,1:100], X[i,101:200])$p.value
	logFC[i] = log2(mean(X[i,1:100]) / mean(X[i,101:200]))
}
E. = data.frame(gnames, P, logFC)
o = order(P)
E = E.[o,]
hist(P)

########################
### Machine Learning ###
########################
### Lineare Diskriminanzanalyse
library(MASS)
k = 10 ### Anzahl an Genen mit denen klassifiziert wird
group = c(rep("control", n1), rep("disease", n2))
X.o = X[o,]
index.train = c(1:50, 101:150) ### Spaltenauswahl für Trainingsdaten
index.test = c(51:100, 151:200) ### Spaltenauswahl für Testdaten
groupx = group[index.train]
M1 = lda(groupx, x=t(X.o[1:k,index.train]))
prediction = predict(M1, newdata=t(X.o[1:k,index.test]))$class

T = table(group[index.test], prediction)

T = table(factor(group[index.test], levels=c("control", "disease")), factor(prediction, levels=c("control", "disease")))

T
accuracy = (T[1,1]+T[2,2]) / sum(T)
binom.test(T[1,1]+T[2,2], sum(T))

sensitivity = (T[2,2]) / sum(T[2,])
binom.test(T[2,2], sum(T[2,]))
specifitiy = (T[1,1]) / sum(T[1,])
binom.test(T[1,1], sum(T[1,]))

PPV = (T[2,2]) / sum(T[,2])
binom.test(T[2,2], sum(T[,2]))
NPV = (T[1,1]) / sum(T[,1])
binom.test(T[1,1], sum(T[,1]))

c(accuracy, sensitivity, specifitiy, PPV, NPV)

### Support Vektor Machines
library(e1071)
k = 5 ### Anzahl an Genen mit denen klassifiziert wird
group = c(rep("control", 100), rep("disease", 100))
group = as.numeric(as.factor(group))-1
X.o = X[o,]
index.train = c(1:50, 101:150) ### Spaltenauswahl für Trainingsdaten
index.test = c(51:100, 151:200) ### Spaltenauswahl für Testdaten
groupx = group[index.train]
M2 = svm(y=groupx, x=t(X.o[1:k,index.train]))
prediction = predict(M2, newdata=t(X.o[1:k,index.test]))

T = table(groupx, prediction>0.5)
T
accuracy = (T[1,1]+T[2,2]) / sum(T)
binom.test(T[1,1]+T[2,2], sum(T))

sensitivity = (T[2,2]) / sum(T[2,])
binom.test(T[2,2], sum(T[2,]))
specifitiy = (T[1,1]) / sum(T[1,])
binom.test(T[1,1], sum(T[1,]))

PPV = (T[2,2]) / sum(T[,2])
binom.test(T[2,2], sum(T[,2]))
NPV = (T[1,1]) / sum(T[,1])
binom.test(T[1,1], sum(T[,1]))

c(accuracy, sensitivity, specifitiy, PPV, NPV)

#########################
#### Aufgabenstellung ###
#########################
#Verändern Sie folgende Parameter und betrachten die Auswirkung auf Accuracy, Sensitivität, Spezifität und Prädiktive Werte (PPV und NPV):
#* Stichprobenumfänge n1 & n2 (bitte auch unbalancierte Szenarien mit n1 deutlich verschieden von n2)
#* Stärke der differentiellen Expression, Parameter sdlog in Zeile 12 (maximal sdlog=0.1)
#* Anzahl k an Genen mit denen klassifiziert wird (Zeile 41 bzw. 67)


















