##############################################
### Generation of Artificial Example Data  ###
##############################################
d = 1000 ### Number of Genes
gnames = paste("Gene.", 1:d)
n1 = 10 ### Number of Samples in Control Group
n2 = 10 ### Number of Samples in Disease Group
n = n1 + n2
mu1 = rlnorm(d, 0, 1)
mu2 = mu1
deg = sample(1:d, d, replace=FALSE)
mu2[deg] = mu2[deg] * rlnorm(d, 0, sdlog=0.05)
hist(log2(mu1/mu2), cex.lab=1.5, cex.axis=1.5, xlab="log2 fold change", ylab="Frequency", main="") ### Verteilung der log2-Fold Changes
X = matrix(NA, d, n)
for (i in 1:n1) X[,i] = rnorm(d, mu1, 0.1 * mu1)
for (i in (n1+1):n) X[,i] = rnorm(d, mu2, 0.1 * mu2)
colnames(X) = c(paste("Control.", 1:10), paste("Disease.", 11:20)) 
heatmap(X)


#########################################
### Differentielle Expressionsanalyse ###
#########################################
P = rep(NA, d)
logFC = rep(NA, d)
for (i in 1:d) {
	if (i%%10==0) print(100 * i/d) 
	P[i] = t.test(X[i,1:10], X[i,11:20])$p.value
	logFC[i] = log2(mean(X[i,1:10]) / mean(X[i,11:20]))
}
E. = data.frame(gnames, P, logFC)
o = order(P)
E = E.[o,]
hist(P)

########################
### Machine Learning ###
########################
### Neuronale Netze
library(neuralnet)
k = 5 ### Anzahl an Genen mit denen klassifiziert wird
group = c(rep("0", 10), rep("1", 10))
group = as.factor(group)
index.train = c(1:5, 11:15) ### Spaltenauswahl für Trainingsdaten
index.test = c(6:10, 16:20) ### Spaltenauswahl für Testdaten
X.o = X[o[1:k],index.train]
groupx = group[index.train]
Z = data.frame(groupx, G1=X.o[1,], G2=X.o[2,], G3=X.o[3,])
M3 = neuralnet(groupx ~ G1 + G2 + G3, data=Z, hidden=c(2, 3), act.fct="logistic")

plot(M3)

X.o = X[o[1:k],index.test]
groupx = group[index.test]
Z = data.frame(G1=X.o[1,], G2=X.o[2,], G2=X.o[3,])

K = predict(M3, newdata=Z)

### Variante 1
predclass = K[,2]>K[,1]
T = table(groupx, predclass)

### Variante 2
predclass = rep(NA, 10)
for (i in 1:10) {
	if (K[i,1]>K[i,2]) predclass[i] = "control"
	if (K[i,1]<K[i,2]) predclass[i] = "diseased"
}
T = table(groupx, predclass)

#########################
#### Aufgabenstellung ###
#########################
# Ermitteln Sie Accuracy, Sensitivität, Spezifität und Prädiktive Werte (PPV und NPV)
# analog zum R-Code für LDA und SVM.
# Verändern Sie wieder folgende Parameter und betrachten die Änderung der Klassifikationsgüte
#* Stichprobenumfänge n1 & n2 (bitte auch unbalancierte Szenarien mit n1 deutlich verschieden von n2)
#* Stärke der differentiellen Expression, Parameter sdlog in Zeile 12 (maximal sdlog=0.1)




















